@@ -1,11 +1,13 @@
-from tkinter.messagebox import NO
 from PyQt6.QtWidgets import QApplication, QCheckBox, QComboBox, QLineEdit, QListWidget, QMainWindow, QMessageBox, QDateEdit, QInputDialog, QPlainTextEdit, QPushButton, QTableWidget, QTableWidgetItem, QRadioButton, QFileDialog
-from .Functions import clear_table_widget, stretch_table_widget_colums_size, list_widget_contains_item, is_url_reachable, get_logical_table_row_list
+from .Functions import clear_table_widget, stretch_table_widget_colums_size, list_widget_contains_item, is_url_reachable, get_logical_table_row_list, create_artifact_source_tag, select_combo_box_data
 from PyQt6.QtCore import Qt, QCoreApplication, QDate
 from .DescriptionWidget import DescriptionWidget
 from .ScreenshotWindow import ScreenshotWindow
+from .RelationsWidget import RelationsWidget
+from .ReleasesWindow import ReleasesWindow
 from .SettingsWindow import SettingsWindow
 from .ValidateWindow import ValidateWindow
+from .ViewXMLWindow import ViewXMLWindow
 from .AboutWindow import AboutWindow
 from .OarsWidget import OarsWidget
 from PyQt6.QtGui import QAction
@@ -30,12 +32,17 @@ class MainWindow(QMainWindow):
 
         self._settings_window = SettingsWindow(env, self)
         self._validate_window = ValidateWindow(env, self)
+        self._xml_window = ViewXMLWindow(env, self)
         self._screenshot_window = ScreenshotWindow(env, self)
+        self._releases_window = ReleasesWindow(env, self)
         self._about_window = AboutWindow(env)
 
         self._description_widget = DescriptionWidget(env, self)
         self.description_layout.addWidget(self._description_widget)
 
+        self._relations_widget = RelationsWidget(env, self)
+        self.relations_layout.addWidget(self._relations_widget)
+
         self._oars_widget = OarsWidget(env, self)
         self.oras_layout.addWidget(self._oars_widget)
 
@@ -66,8 +73,21 @@ class MainWindow(QMainWindow):
 
         self._update_recent_files_menu()
 
-        for i in env.license_list["licenses"]:
-            self.metadata_license_box.addItem(i["name"], i["licenseId"])
+        self.component_type_box.addItem(QCoreApplication.translate("MainWindow", "Desktop"), "desktop")
+        self.component_type_box.addItem(QCoreApplication.translate("MainWindow", "Console"), "console-application")
+        self.component_type_box.addItem(QCoreApplication.translate("MainWindow", "Web Application"), "web-application")
+        self.component_type_box.addItem(QCoreApplication.translate("MainWindow", "Service"), "service")
+        self.component_type_box.addItem(QCoreApplication.translate("MainWindow", "Addon"), "addon")
+        self.component_type_box.addItem(QCoreApplication.translate("MainWindow", "Font"), "font")
+        self.component_type_box.addItem(QCoreApplication.translate("MainWindow", "Icon Theme"), "icon-theme")
+        self.component_type_box.addItem(QCoreApplication.translate("MainWindow", "Codecs"), "codec")
+        self.component_type_box.addItem(QCoreApplication.translate("MainWindow", "Input Method"), "inputmethod")
+        self.component_type_box.addItem(QCoreApplication.translate("MainWindow", "Firmware"), "firmware")
+
+        for key, value in env.metadata_license_list.items():
+            self.metadata_license_box.addItem(value, key)
+
+        for i in env.project_license_list["licenses"]:
             self.project_license_box.addItem(i["name"], i["licenseId"])
 
         self.metadata_license_box.model().sort(0, Qt.SortOrder.AscendingOrder)
@@ -129,7 +149,9 @@ class MainWindow(QMainWindow):
         self.exit_action.triggered.connect(self._exit_menu_action_clicked)
 
         self.settings_action.triggered.connect(self._settings_window.open_window)
+
         self.validate_action.triggered.connect(self._validate_window.open_window)
+        self.view_xml_action.triggered.connect(self._xml_window.exec)
 
         self.documentation_action.triggered.connect(lambda: webbrowser.open("https://www.freedesktop.org/software/appstream/docs"))
         self.about_action.triggered.connect(self._about_window.exec)
@@ -205,6 +227,7 @@ class MainWindow(QMainWindow):
             return
         self.reset_data()
         self._edited = False
+        self._current_path = None
         self.update_window_title()
 
     def _open_menu_action_clicked(self):
@@ -215,7 +238,6 @@ class MainWindow(QMainWindow):
             return
         self.open_file(path[0])
         self._add_to_recent_files(path[0])
-        self._current_path = path[0]
         self.update_window_title()
 
     def _open_recent_file(self):
@@ -226,7 +248,6 @@ class MainWindow(QMainWindow):
             return
         self.open_file(action.data())
         self._add_to_recent_files(action.data())
-        self._current_path = action.data()
         self.update_window_title()
 
     def _save_file_clicked(self):
@@ -331,8 +352,13 @@ class MainWindow(QMainWindow):
 
     # Releases
 
-    def _set_release_row(self, row: int, version: Optional[str] = "", date: Optional[QDate] = None, development: bool = False):
-        self.releases_table.setItem(row, 0, QTableWidgetItem(version))
+    def _set_release_row(self, row: int, version: Optional[str] = "", date: Optional[QDate] = None, development: bool = False, data: Optional[dict] = None):
+        version_item = QTableWidgetItem(version)
+        if data:
+            version_item.setData(42, data)
+        else:
+            version_item.setData(42, {})
+        self.releases_table.setItem(row, 0, version_item)
 
         if date is None:
             self.releases_table.setCellWidget(row, 1, QDateEdit())
@@ -346,13 +372,22 @@ class MainWindow(QMainWindow):
             type_box.setCurrentIndex(1)
         self.releases_table.setCellWidget(row, 2, type_box)
 
+        edit_button = QPushButton(QCoreApplication.translate("MainWindow", "Edit"))
+        edit_button.clicked.connect(self._release_edit_button_clicked)
+        self.releases_table.setCellWidget(row, 3, edit_button)
+
         remove_button = QPushButton(QCoreApplication.translate("MainWindow", "Remove"))
         remove_button.clicked.connect(self._release_remove_button_clicked)
-        self.releases_table.setCellWidget(row, 3, remove_button)
+        self.releases_table.setCellWidget(row, 4, remove_button)
+
+    def _release_edit_button_clicked(self):
+        for i in range(self.releases_table.rowCount()):
+            if self.releases_table.cellWidget(i, 3) == self.sender():
+                self._releases_window.open_window(i)
 
     def _release_remove_button_clicked(self):
         for i in range(self.releases_table.rowCount()):
-            if self.releases_table.cellWidget(i, 3) == self.sender():
+            if self.releases_table.cellWidget(i, 4) == self.sender():
                 self.releases_table.removeRow(i)
                 self.set_file_edited()
                 return
@@ -384,8 +419,16 @@ class MainWindow(QMainWindow):
                 return
             clear_table_widget(self.releases_table)
         for count, i in enumerate(api_data):
+            data = {}
+            data["url"] = i["html_url"]
+            # description_tag = etree.Element("description")
+            # paragraph_tag = etree.SubElement(description_tag, "p")
+            # paragraph_tag.text = i["body"]
+            # data["description"] = description_tag
+
+            #tarball_url =
             self.releases_table.insertRow(count)
-            self._set_release_row(count, version = i["tag_name"], date=QDate.fromString(i["published_at"], Qt.DateFormat.ISODate), development=i["prerelease"])
+            self._set_release_row(count, version = i["tag_name"], date=QDate.fromString(i["published_at"], Qt.DateFormat.ISODate), development=i["prerelease"], data=data)
         self.set_file_edited()
 
     def _release_import_gitlab(self):
@@ -406,8 +449,18 @@ class MainWindow(QMainWindow):
                 return
             clear_table_widget(self.releases_table)
         for count, i in enumerate(r.json()):
+            data = {}
+            data["url"] = i["_links"]["self"]
+
+            for source in i["assets"]["sources"]:
+                if source["format"] == "tar.gz":
+                    artifacts_tag = etree.Element("artifacts")
+                    artifacts_tag.append(create_artifact_source_tag(source["url"]))
+                    data["artifacts"] = artifacts_tag
+                    break
+
             self.releases_table.insertRow(count)
-            self._set_release_row(count, version = i["name"], date=QDate.fromString(i["released_at"], Qt.DateFormat.ISODate))
+            self._set_release_row(count, version = i["name"], date=QDate.fromString(i["released_at"], Qt.DateFormat.ISODate), data=data)
         self.set_file_edited()
 
     def _check_links_url_button_clicked(self):
@@ -550,12 +603,15 @@ class MainWindow(QMainWindow):
                 value.clear()
         self._description_widget.reset_data()
         self.screenshot_list.clear()
+        self._relations_widget.reset_data()
         self._oars_widget.reset_data()
         self._name_translations.clear()
         self._summary_translations.clear()
         self._update_categorie_remove_button_enabled()
         self._update_keyword_edit_remove_button()
 
+    # Read
+
     def _parse_screenshots_tag(self, screenshots_tag: etree._Element):
         for i in screenshots_tag.getchildren():
             new_dict = {}
@@ -573,7 +629,6 @@ class MainWindow(QMainWindow):
                 continue
 
             image_tag = i.find("image")
-            new_dict["type"] = image_tag.get("type", "source")
             new_dict["url"] = image_tag.text
 
             width = image_tag.get("width")
@@ -606,6 +661,15 @@ class MainWindow(QMainWindow):
 
         self.reset_data()
 
+        if len(root.xpath("/component")) == 0:
+            QMessageBox.critical(self, QCoreApplication.translate("MainWindow", "No component tag"), QCoreApplication.translate("MainWindow", "This XML file has no component tag"))
+            return
+        elif len(root.xpath("/component")) > 2:
+            QMessageBox.critical(self, QCoreApplication.translate("MainWindow", "Too many component tag"), QCoreApplication.translate("MainWindow", "Only files with one component tag are supported"))
+            return
+
+        select_combo_box_data(self.component_type_box, root.xpath("/component")[0].get("type"))
+
         id_tag = root.find("id")
         if id_tag is not None:
             self.id_edit.setText(id_tag.text)
@@ -662,8 +726,20 @@ class MainWindow(QMainWindow):
         if releases_tag is not None:
             for i in releases_tag.getchildren():
                 current_row = self.releases_table.rowCount()
+                data = {}
+                if i.get("urgency") is not None:
+                    data["urgency"] = i.get("urgency")
+                url_tag = i.find("url")
+                if url_tag is not None:
+                    data["url"] = url_tag.text
+                description_tag = i.find("description")
+                if description_tag is not None:
+                    data["description"] = description_tag
+                artifacts_tag = i.find("artifacts")
+                if artifacts_tag is not None:
+                     data["artifacts"] = artifacts_tag
                 self.releases_table.insertRow(current_row)
-                self._set_release_row(current_row, version=i.get("version"), date=QDate.fromString(i.get("date"), Qt.DateFormat.ISODate), development=(i.get("type") == "development"))
+                self._set_release_row(current_row, version=i.get("version"), date=QDate.fromString(i.get("date"), Qt.DateFormat.ISODate), development=(i.get("type") == "development"), data=data)
 
         categories_tag = root.find("categories")
         if categories_tag is not None:
@@ -672,7 +748,7 @@ class MainWindow(QMainWindow):
 
         for i in root.findall("url"):
             try:
-                getattr(self, i.get("type") + "_url_edit").setText(i.text)
+                getattr(self, i.get("type").replace("-", "_") + "_url_edit").setText(i.text)
             except AttributeError:
                 print(f"Unknown URL type {i.get('type')}", file=sys.stderr)
 
@@ -687,6 +763,7 @@ class MainWindow(QMainWindow):
                     box.setCurrentIndex(index)
                 except AttributeError:
                     print(f"Unknown value {i.text} for control tag")
+            self._relations_widget.load_data(current_tag)
 
         content_rating_tag = root.find("content_rating")
         if content_rating_tag is not None:
@@ -712,18 +789,49 @@ class MainWindow(QMainWindow):
 
         self._edited = False
 
+        self._current_path = path
+
+    # Write
+
+    def _write_releases(self, root_tag: etree.Element):
+        releases_tag = etree.SubElement(root_tag, "releases")
+        for i in get_logical_table_row_list(self.releases_table):
+            version = self.releases_table.item(i, 0).text()
+            date = self.releases_table.cellWidget(i, 1).date().toString(Qt.DateFormat.ISODate)
+            release_type = self.releases_table.cellWidget(i, 2).currentData()
+            single_release_tag = etree.SubElement(releases_tag, "release")
+            single_release_tag.set("version", version)
+            single_release_tag.set("date", date)
+            single_release_tag.set("type", release_type)
+
+            data = self.releases_table.item(i, 0).data(42)
+
+            if "urgency" in data:
+                single_release_tag.set("urgency", data["urgency"])
+
+            if "url" in data:
+                url_tag =  etree.SubElement(single_release_tag, "url")
+                url_tag.text = data["url"]
+
+            if "description" in data:
+                single_release_tag.append(data["description"])
+
+            if "artifacts" in data:
+                single_release_tag.append(data["artifacts"])
+
     def _write_requires_recommends_supports_tags(self, root_tag: etree._Element, current_type: str):
         current_tag = etree.SubElement(root_tag, current_type)
         for i in self._control_type_list:
             if getattr(self, "control_box_" + i).currentData() == current_type:
                 control_tag = etree.SubElement(current_tag, "control")
                 control_tag.text = i.replace("_", "-") # For tv-remote - is in a object name not supportet
+        self._relations_widget.get_save_data(current_tag, current_type)
         if len(current_tag.getchildren()) == 0:
             root_tag.remove(current_tag)
 
-    def save_file(self, path: str):
+    def get_xml_text(self) -> str:
         root = etree.Element("component")
-        root.set("type", "desktop")
+        root.set("type", self.component_type_box.currentData())
 
         if self._env.settings.get("addCommentSave"):
             root.append(etree.Comment("Created with jdAppdataEdit " + self._env.version))
@@ -787,29 +895,21 @@ class MainWindow(QMainWindow):
                     caption_trans_tag.text = value
                 image_tag = etree.SubElement(single_screenshot_tag, "image")
                 image_tag.text = i["url"]
-                image_tag.set("type", i["type"])
+                image_tag.set("type", "source")
                 if "width" in i:
                     image_tag.set("width", str(i["width"]))
                 if "height" in i:
                     image_tag.set("height", str(i["height"]))
 
         if self.releases_table.rowCount() > 0:
-            releases_tag = etree.SubElement(root, "releases")
-            for i in get_logical_table_row_list(self.releases_table):
-                version = self.releases_table.item(i, 0).text()
-                date = self.releases_table.cellWidget(i, 1).date().toString(Qt.DateFormat.ISODate)
-                release_type = self.releases_table.cellWidget(i, 2).currentData()
-                single_release_tag = etree.SubElement(releases_tag, "release")
-                single_release_tag.set("version", version)
-                single_release_tag.set("date", date)
-                single_release_tag.set("type", release_type)
+            self._write_releases(root)
 
         for i in self._url_list:
             url = getattr(self, f"{i}_url_edit").text()
             if url == "":
                 continue
             url_tag = etree.SubElement(root, "url")
-            url_tag.set("type", i)
+            url_tag.set("type", i.replace("_", "-"))
             url_tag.text = url
 
         if self.categorie_list.count() > 0:
@@ -852,8 +952,11 @@ class MainWindow(QMainWindow):
         xml = xml.replace("&lt;", "<")
         xml = xml.replace("&gt;", ">")
 
+        return xml
+
+    def save_file(self, path: str):
         with open(path, "w", encoding="utf-8", newline='\n') as f:
-            f.write(xml)
+            f.write(self.get_xml_text())
 
     def closeEvent(self, event):
         if self._ask_for_save():
